﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Mvc;
using EJ2ScheduleSample.Models;
using EJ2ScheduleSample.Data;
using Newtonsoft.Json;
using System.Collections;

namespace EJ2ScheduleSample.Controllers
{
    public class HomeController : Controller
    {
        private ScheduleDataContext _context;
        public IActionResult Index()
        {

            List<ResourceDataSourceModel> projects = new List<ResourceDataSourceModel>();
            projects.Add(new ResourceDataSourceModel { text = "PROJECT 1", id = 1, color = "#cb6bb2" });
            projects.Add(new ResourceDataSourceModel { text = "PROJECT 2", id = 2, color = "#56ca85" });
            ViewBag.Projects = projects;

            List<ResourceDataSourceModel> categories = new List<ResourceDataSourceModel>();
            categories.Add(new ResourceDataSourceModel { text = "Development", id = 1, color = "#df5286" });
            categories.Add(new ResourceDataSourceModel { text = "Testing", id = 2, color = "#7fa900" });
            ViewBag.Categories = categories;

            ViewBag.Resources = new string[] { "Projects", "Categories" };
            return View();
        }
        class ResourceDataSourceModel
        {
            public int id { set; get; }
            public string text { set; get; }
            public string color { set; get; }
        }
      

        [HttpPost]
        public List<ScheduleEvent> LoadData([FromBody]Params param)
        {
            var data = _context.ScheduleEvents.ToList();

            DateTime startdate = DateTime.Today;
            DateTime enddate = startdate.AddDays(1);
            List<ScheduleEvent> appCollection = new List<ScheduleEvent>();
            for (var i =0; i< data.Count; i++)
            {
                var app = data[i];
                if (data[i].RecurrenceRule !=  null) {  
                    TimeSpan diff = app.EndTime.Subtract(app.StartTime);
                    var recurrenceRule = data[i].RecurrenceRule;
                    TimeSpan startDiff;
                    var dateCollection = RecurrenceHelper.GetRecurrenceDateTimeCollection(recurrenceRule, DateTime.Now);
                    foreach (var date in dateCollection)
                    {
                        startDiff = app.StartTime.Subtract(date);
                        break;
                    }
                        foreach (var date in dateCollection)
                    {
                        var start = date.Add(startDiff);
                        var end = start.Add(diff);
                        appCollection.Add(new ScheduleEvent {  Subject=app.Subject, StartTime = start, EndTime = end, CategoryId=app.CategoryId, Description = app.Description, IsAllDay =app.IsAllDay, RecurrenceRule = app.RecurrenceRule, Location = app.Location,ProjectId = app.ProjectId, StartTimezone = app.StartTimezone, EndTimezone = app.EndTimezone,Id = app.Id });
                    }
                }
                else
                {
                    appCollection.Add(app);
                }
            }
            // Here we fileter the today appointment from the db
            var todayAppointmentCollection = appCollection.Where(app => (app.StartTime >= startdate && app.StartTime <= enddate)).ToList();  
            Console.WriteLine(todayAppointmentCollection); // Now the today appointment available in 'todayAppointemntCollection variable.'
            return data;
        }
       

        public class Params{
            public string StartDate { get;  set; }
            public string EndDate { get; set; }
        }

        [HttpPost]
        public List<ScheduleEvent> UpdateData([FromBody]EditParams param)
        {
            if (param.action == "insert" || (param.action == "batch" && param.added.Count > 0)) // this block of code will execute while inserting the appointments
            {
                var value = (param.action == "insert") ? param.value : param.added[0];
                int intMax = _context.ScheduleEvents.ToList().Count > 0 ? _context.ScheduleEvents.ToList().Max(p => p.Id) : 1;
                DateTime startTime = Convert.ToDateTime(value.StartTime);
                DateTime endTime = Convert.ToDateTime(value.EndTime);
                ScheduleEvent appointment = new ScheduleEvent()
                {
                    StartTime = startTime.ToLocalTime(),
                    EndTime = endTime.ToLocalTime(),
                    Subject = value.Subject,
                    IsAllDay = value.IsAllDay,
                    StartTimezone = value.StartTimezone,
                    EndTimezone = value.EndTimezone,
                    RecurrenceRule = value.RecurrenceRule,
                    RecurrenceID = value.RecurrenceID,
                    RecurrenceException = value.RecurrenceException,
                    Description = value.Description,
                    CategoryId = value.CategoryId,
                    ProjectId = value.ProjectId
                };
                _context.ScheduleEvents.Add(appointment);
                _context.SaveChanges();
            }
            if (param.action == "update" || (param.action == "batch" && param.changed.Count > 0)) // this block of code will execute while removing the appointment
            {
                var value = (param.action == "update") ? param.value : param.changed[0];
                var filterData = _context.ScheduleEvents.Where(c => c.Id == Convert.ToInt32(value.Id));
                if (filterData.Count() > 0)
                {
                    DateTime startTime = Convert.ToDateTime(value.StartTime);
                    DateTime endTime = Convert.ToDateTime(value.EndTime);
                    ScheduleEvent appointment = _context.ScheduleEvents.Single(A => A.Id == Convert.ToInt32(value.Id));
                    appointment.StartTime = startTime.ToLocalTime();
                    appointment.EndTime = endTime.ToLocalTime();
                    appointment.StartTimezone = value.StartTimezone;
                    appointment.EndTimezone = value.EndTimezone;
                    appointment.Subject = value.Subject;
                    appointment.IsAllDay = value.IsAllDay;
                    appointment.RecurrenceRule = value.RecurrenceRule;
                    appointment.RecurrenceID = value.RecurrenceID;
                    appointment.RecurrenceException = value.RecurrenceException;
                    appointment.Description = value.Description;
                    appointment.ProjectId = value.ProjectId;
                    appointment.CategoryId = value.CategoryId;
                }
                _context.SaveChanges();
            }
            if (param.action == "remove" || (param.action == "batch" && param.deleted.Count > 0)) // this block of code will execute while updating the appointment
            {
                if (param.action == "remove")
                {
                    int key = Convert.ToInt32(param.key);
                    ScheduleEvent appointment = _context.ScheduleEvents.Where(c => c.Id == key).FirstOrDefault();
                    if (appointment != null) _context.ScheduleEvents.Remove(appointment);
                }
                else
                {
                    foreach (var apps in param.deleted)
                    {
                        ScheduleEvent appointment = _context.ScheduleEvents.Where(c => c.Id == apps.Id).FirstOrDefault();
                        if (apps != null) _context.ScheduleEvents.Remove(appointment);
                    }
                }
                _context.SaveChanges();
            }
            return _context.ScheduleEvents.ToList();
        }


        public HomeController(ScheduleDataContext context)
        {
            _context = context;
        }

        public IActionResult About()
        {
            ViewData["Message"] = "Your application description page.";

            return View();
        }

        public IActionResult Contact()
        {
            ViewData["Message"] = "Your contact page.";

            return View();
        }

        public IActionResult Error()
        {
            return View(new ErrorViewModel { RequestId = Activity.Current?.Id ?? HttpContext.TraceIdentifier });
        }
    }
}
